/*
 * Copyright 2013 Robert von Burg <eitch@eitchnet.ch>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package li.strolch.command.privilege.users;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import java.util.*;

import li.strolch.command.AbstractRealmCommandTest;
import li.strolch.persistence.api.StrolchTransaction;
import li.strolch.privilege.model.UserRep;
import li.strolch.privilege.model.UserState;
import li.strolch.service.api.Command;
import li.strolch.service.privilege.users.PrivilegeAddUserCommand;

/**
 * @author Robert von Burg <eitch@eitchnet.ch>
 */
public class PrivilegeAddUserCommandTest extends AbstractRealmCommandTest {

	@Override
	protected String getUsername() {
		return "admin";
	}

	@Override
	protected Command getCommandInstance(StrolchTransaction tx) {

		Set<String> roles = new HashSet<>();
		roles.add("AppUser");
		Map<String, String> propertyMap = new HashMap<>();

		UserRep user = new UserRep(null, "dude", "Jeff", "Lebowski", UserState.ENABLED, roles, Locale.getDefault(),
				propertyMap, null);

		PrivilegeAddUserCommand command = new PrivilegeAddUserCommand(tx);
		command.setUserIn(user);
		return command;
	}

	@Override
	protected void validateAfterCommand(StrolchTransaction tx) {
		assertNotNull(tx.getContainer().getPrivilegeHandler().getPrivilegeHandler().getUser(certificate, "dude"));
	}

	@Override
	protected void validateAfterCommandFailed(StrolchTransaction tx) {
		assertNull(tx.getContainer().getPrivilegeHandler().getPrivilegeHandler().getUser(certificate, "dude"));
	}
}
